cdef extern from "chicken.h":
    void *CHICKEN_default_toplevel
    long CHICKEN_run (void *)
    int CHICKEN_eval_string_to_string (char *, char *, int)
    
cdef int STRING_LENGTH = 255
cdef char * C_chick_eval(char *in_str):
    # The size of OUT_STRING should be STRING_LENGTH + 1
    # Cython complains if I put in that expession though
    cdef char[256] OUT_STRING 
    status = CHICKEN_eval_string_to_string(in_str, OUT_STRING , STRING_LENGTH)
    if (status == 0):
        print "Chicken evaluation failed\n"
    print OUT_STRING
    return OUT_STRING


#------------------------------------------------------------#
#                   Function definitions                     #
#------------------------------------------------------------#

LINE_NUMBER = 1
C_DICTIONARY = {}
SETUP_DONE = 0

def inc_line_number ():
    global LINE_NUMBER
    LINE_NUMBER += 1

def reset_line_number ():
    global LINE_NUMBER
    LINE_NUMBER = 1
    
def inc_setup_done ():
    global SETUP_DONE
    SETUP_DONE = 1

def chick_eval(in_str):
    return C_chick_eval(in_str)

def c_eval (prompt="#;" + str(LINE_NUMBER) + "> "):
    """
    Evaluates a Chicken Scheme expression.
    """
    global LINE_NUMBER
    global SETUP_DONE
    if (SETUP_DONE == 0):
        c_setup ()
        inc_setup_done ()
    in_str = list()
    hold = ""
    while True:
        hold_str1 = raw_input (prompt)
        tester = ("".join (hold_str1.split ()))
        # Evaluate the Scheme expression using a newline
        if (tester == ""):
            eval_str = " ".join (in_str)          
            hold = chick_eval (eval_str)
            inc_line_number ()
            prompt="#;" + str(LINE_NUMBER) + "> "
            in_str = list()
        # Exit the Scheme REPL\
        elif (tester [-6:] == "(exit)"):
            print "\nProcess scheme finished"
            reset_line_number ()
            return hold
        # A multi-line expression
        else:
            hold_str2 = parse (hold_str1)
            # A keyword is not in the Cython-Python dictionary
            if (check_str (hold_str2)):
                prompt="#;" + str(LINE_NUMBER) + "> "
                in_str = list()
            else:
                in_str.append (hold_str2)
                prompt = "  "

def c_setup ():
    """
    Set up the Chicken Scheme environment.
    """
    global SETUP_DONE
    CHICKEN_run (CHICKEN_default_toplevel)
    import string
    inc_setup_done ()

def c_dict_add (val_name, val):
    """
    Puts a new key-value pair in the Chicken-Python dictionary.
    """
    global C_DICTIONARY
    C_DICTIONARY [val_name] = val

def c_dict_display ():
    """
    Display the Chicken-Python dictionary.
    """
    return C_DICTIONARY

def c_dict_clear ():
    """
    Clears the Chicken-Python dictionary.
    """
    global C_DICTIONARY
    C_DICTIONARY = {}

def parse (in_str):
    """
    Replaces keywords with their dictionary values.
    """
    global C_DICTIONARY
    import re
    try:
        indx = re.search (';.*?;', in_str).span ()        
    except:
        return in_str    
    indx1 = min (indx) + 1
    indx2 = max (indx) - 1    
    # The key isn't in the dictionary
    if not C_DICTIONARY.has_key (in_str [indx1:indx2]):
        return in_str [indx1 - 1:indx2 + 1]
    in_str = in_str [:indx1 - 1] + " " + \
              str (C_DICTIONARY [in_str [indx1:indx2]]) \
              + " " + parse (in_str [indx2 + 1:])
    return in_str

def check_str (in_str):
    import re
    try:
        indx = re.search (';.*?;', in_str).span ()
        indx1 = min (indx) + 1
        indx2 = max (indx) - 1
        print "Key " + in_str[indx1:indx2] + " is not in dictionary."
        return True
    except:
        return False

        

